/****************************************************************************
 *
 * This file is part of the ustk software.
 * Copyright (C) 2016 - 2017 by Inria. All rights reserved.
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * ("GPL") version 2 as published by the Free Software Foundation.
 * See the file LICENSE.txt at the root directory of this source
 * distribution for additional information about the GNU GPL.
 *
 * For using ustk with software that can not be combined with the GNU
 * GPL, please contact Inria about acquiring a ViSP Professional
 * Edition License.
 *
 * This software was developed at:
 * Inria Rennes - Bretagne Atlantique
 * Campus Universitaire de Beaulieu
 * 35042 Rennes Cedex
 * France
 *
 * If you have questions regarding the use of this file, please contact
 * Inria at ustk@inria.fr
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Authors:
 * Marc Pouliquen
 *
 *****************************************************************************/

#include "usConsoleListener.h"
#include <iostream>

/**
 * Constructor.
 */
usConsoleListener::usConsoleListener()
{
#if defined(_WIN32)
  m_notifier = new QSocketNotifier(_fileno(stdin), QSocketNotifier::Read, this);
#else
  m_notifier = new QSocketNotifier(fileno(stdin), QSocketNotifier::Read, this);
#endif
  connect(m_notifier, SIGNAL(activated(int)), this, SLOT(readCommand()));
}

/**
 * Slot Called when an input is detected in console.
 * If it's "p" character, it emits quitPause signal.
 */
void usConsoleListener::readCommand()
{
  std::string line;
  std::getline(std::cin, line);
  if (line == "p") {
    emit quitPause();
  } else {
    std::cout << "Unknow console command : " << line << std::endl << std::flush;
  }
}
